--[[
	Project: CombatAlerts
	Author: TimeBomb

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

]]

-- initialise the Alert textbox
if not CAlerts then CAlerts = {} end
CAlerts.Alert = class()
CAlerts.AlertInstance = nil
function CAlerts.Alert:init(context)

  -- we use a config variable to decide whether to show alerts
  if not CAlerts_CONFIG then CAlerts_CONFIG = {} end
  if not CAlerts_CONFIG['showAlerts'] then CAlerts_CONFIG['showAlerts'] = true end

   -- @field create the context
  self.context = UI.CreateContext("CAlertsUIContext")
  self.alertTypeFrame = UI.CreateFrame("Text", "CAlertsUISummaryLifetime", self.context)
  self.alertTypeFrame:SetPoint("CENTER", UIParent, "CENTER", 0, -300)
  self.alertTypeFrame:SetText("")
  self.alertTypeFrame:SetFont("CAlerts", CAlerts_Config["fonttitle"])
  self.alertTypeFrame:SetFontSize(65)
  self.alertTypeFrame:SetFontColor(1,0,0,1)
  self.alertTypeFrame:SetVisible(true)

  self.alertMessageFrame = UI.CreateFrame("Text", "CAlertsUISummaryLifetime", self.context)
  self.alertMessageFrame:SetPoint("CENTER", UIParent, "CENTER", 0, -200)
  self.alertMessageFrame:SetText("")
  self.alertMessageFrame:SetFont("CAlerts", CAlerts_Config["fontmessage"])
  self.alertMessageFrame:SetFontSize(40)
  self.alertMessageFrame:SetFontColor(1,215/255,0,1)
  self.alertMessageFrame:SetVisible(true)

  CAlerts.AlertInstance = self

  self.icons = {}
  self.icons['SoloKill']= self:createIcon(-150, -250, "images/solokill_1.png", "images/solokill_2.png")
  self.icons['KillingBlow']= self:createIcon(-75, -250, "images/killingblow_1.png", "images/killingblow_2.png")
  self.icons['DamageAssist']= self:createIcon(0, -250, "images/damage_dealer_1.png", "images/damage_dealer_2.png")
  self.icons['DistractAssist']= self:createIcon(75, -250, "images/defender_1.png","images/defender_2.png")
  self.icons['HealAssist']= self:createIcon(150, -250, "images/healer_1.png", "images/healer_2.png")

  self.lastCall=0
  self.madeVisible=0
end

-- Creates a new icon frame, stores and sets its texture and returns the object
-- appends two new functions to the icon frame: enable and disable that toggle the texture.
-- @param dx the x offset from the center of UIParent
-- @param dx the y offset from the center of UIParent
-- @param tex1 the disabled texture
-- @param tex2 the enabled texture
function CAlerts.Alert:createIcon(dx, dy, tex1, tex2)
  local icon = UI.CreateFrame("Texture", "CAlertsAlertIcon", self.context)
  function icon:enable()
    self:SetTexture("CAlerts", tex2)
  end
   function icon:disable()
    self:SetTexture("CAlerts", tex1)
  end
  icon:SetTexture("CAlerts", tex1)
  icon:ResizeToTexture()
  icon:SetPoint("CENTER", UIParent, "CENTER", dx, dy)
  icon:SetLayer(10)
  icon:SetVisible(false)
  return icon
end

function CAlerts.Alert:setMessage(title, msg, flags)

  if CAlerts_CONFIG.toggle == 0 then -- Turned off...
    return
  end

  -- Just some code to stop possible error messages
  if title == nil then
    title = "NIL"
  end
  if msg == nil then
    msg = "NIL"
  end

  if not CAlerts_CONFIG['showAlerts'] then
    return
  end

  self.alertTypeFrame:SetText(title)
  self.alertMessageFrame:SetText(msg)

  self.alertTypeFrame:SetPoint("CENTER", UIParent, "CENTER", 0, -300)
  self.alertMessageFrame:SetPoint("CENTER", UIParent, "CENTER", 0, -200)

  self.alertTypeFrame:SetVisible(true)
  self.alertMessageFrame:SetVisible(true)

  -- set icons
  if flags then
    for i,v in pairs(self.icons) do
      if table.contains(flags, i) then v:enable() else v:disable() end
      v:SetVisible(true)
    end
  end

  self.madeVisible = Inspect.Time.Real()
end

local lastCall = Inspect.Time.Real()
function CAlerts.Alert:Update()

  -- get reference to self
  local obj = CAlerts.AlertInstance

  -- we only want this running once every 10th of a second or we'll
  -- break someones CPU
  local now = Inspect.Time.Real()
  if (now<obj.lastCall+0.1) then
      return
  end
  obj.lastCall = now

  local delta = now-obj.madeVisible
  if delta<5 then
    obj.alertTypeFrame:SetVisible(true)
    obj.alertTypeFrame:SetAlpha((5-delta)/(5))
    obj.alertMessageFrame:SetVisible(true)
    obj.alertMessageFrame:SetAlpha((5-delta)/(5))

    for i,v in pairs(obj.icons) do
      v:SetVisible(true)
      v:SetAlpha((5-delta)/(5))
    end
  else
    obj.alertTypeFrame:SetVisible(false)
    obj.alertTypeFrame:SetAlpha(1)
    obj.alertMessageFrame:SetVisible(false)
    obj.alertMessageFrame:SetAlpha(1)

    for i,v in pairs(obj.icons) do
      v:SetVisible(false)
      v:SetAlpha(1)
    end
  end

end

